﻿using System;
using System.Collections.Generic;
using System.Linq;

namespace UMK.WzorceStrukturalne
{
    public interface IUrządzenieRTV
    {
        void Włącz();
        void Wyłącz();
        void UstawKanał(int numerKanału);
    }

    public class Telewizor : IUrządzenieRTV
    {
        public void Włącz()
        {
            Console.WriteLine("Telewizor zostal wlaczony");
        }
        public void Wyłącz()
        {
            Console.WriteLine("Telewizor zostal wylaczony");
        }
        public void UstawKanał(int numerKanału)
        {
            Console.WriteLine("Kanal w telewizorze zmieniony na: " + numerKanału);
        }
    }

    public class Radio : IUrządzenieRTV
    {
        public void Włącz()
        {
            Console.WriteLine("Radio zostalo wlaczone");
        }
        public void Wyłącz()
        {
            Console.WriteLine("Radio zostalo wylaczone");
        }
        public void UstawKanał(int numerKanału)
        {
            Console.WriteLine("Kanal w radiu zmieniony na: " + numerKanału);
        }
    }

    public class PilotUniwersalny
    {
        private IUrządzenieRTV urządzenie;

        private int poprzedniKanał, bieżącyKanał;

        public PilotUniwersalny(IUrządzenieRTV urządzenie)
        {
            this.urządzenie = urządzenie;
        }

        public void Włącz()
        {
            urządzenie.Włącz();
            UstawKanał(1);
        }

        public void UstawKanał(int numerKanału)
        {
            poprzedniKanał = bieżącyKanał;
            bieżącyKanał = numerKanału;
            urządzenie.UstawKanał(bieżącyKanał);
        }

        public void Wyłącz()
        {
            urządzenie.Wyłącz();
        }

        public void PrzywróćPoprzedniKanał()
        {
            UstawKanał(poprzedniKanał);
        }
    }

    public class PilotZespolony
    {
        private List<IUrządzenieRTV> urządzenia = new List<IUrządzenieRTV>();

        private int poprzedniKanał, bieżącyKanał;

        public PilotZespolony(params IUrządzenieRTV[] urządzenia)
        {
            this.urządzenia = urządzenia.ToList();
        }

        public void DodajUrządzenie(IUrządzenieRTV urządzenie)
        {
            urządzenia.Add(urządzenie);
        }

        public void UsuńUrządzenie(IUrządzenieRTV urządzenie)
        {
            urządzenia.Remove(urządzenie);
        }

        public void Włącz()
        {
            foreach(IUrządzenieRTV urządzenie in urządzenia)
                urządzenie.Włącz();
            UstawKanał(1);
        }

        public void UstawKanał(int numerKanału)
        {
            poprzedniKanał = bieżącyKanał;
            bieżącyKanał = numerKanału;
            foreach (IUrządzenieRTV urządzenie in urządzenia)
                urządzenie.UstawKanał(bieżącyKanał);
        }

        public void Wyłącz()
        {
            foreach (IUrządzenieRTV urządzenie in urządzenia)
                urządzenie.Wyłącz();
        }

        public void PrzywróćPoprzedniKanał()
        {
            UstawKanał(poprzedniKanał);
        }
    }
}
